@@ -1,4 +1,4 @@
-# Copyright (c) 2017, Riverbank Computing Limited
+# Copyright (c) 2020, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,12 +24,13 @@
 # POSSIBILITY OF SUCH DAMAGE.
 
 
-from PyQt5.QtCore import pyqtSignal, Qt
-from PyQt5.QtWidgets import (QButtonGroup, QCheckBox, QComboBox, QFileDialog,
-        QGridLayout, QGroupBox, QHBoxLayout, QLineEdit, QRadioButton, QWidget)
+from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QFontInfo, QGuiApplication
+from PyQt5.QtWidgets import (QCheckBox, QFileDialog, QGridLayout, QGroupBox,
+        QLineEdit, QPlainTextEdit, QWidget)
 
-from ..metadata import supported_python_versions
 from .better_form import BetterForm
+from .collapsible_widget import CollapsibleWidget
 from .filename_editor import FilenameEditor
 from .package_editor import PackageEditor
 
@@ -40,12 +41,6 @@ class ApplicationPage(QWidget):
     # The page's label.
     label = "Application Source"
 
-    # Emitted when the user changes the PyQt version.
-    pyqt_version_changed = pyqtSignal(bool)
-
-    # Emitted when the user changes the Python target version.
-    python_target_version_changed = pyqtSignal()
-
     @property
     def project(self):
         """ The project property getter. """
@@ -109,22 +104,6 @@ class ApplicationPage(QWidget):
 
         options_layout = BetterForm()
 
-        self._py_version_edit = QComboBox(
-                whatsThis="Select the target Python version.")
-        for major, minor, patch in supported_python_versions:
-            self._py_version_edit.addItem(
-                    "v{0}.{1}.{2}".format(major, minor, patch))
-        self._py_version_edit.currentIndexChanged.connect(
-                self._py_version_changed)
-        options_layout.addRow("Target Python version", self._py_version_edit)
-
-        self._pyqt_version_edit = QComboBox(
-                whatsThis="Select the PyQt version.")
-        self._pyqt_version_edit.addItems(["PyQt4", "PyQt5"])
-        self._pyqt_version_edit.currentIndexChanged.connect(
-                self._pyqt_version_changed)
-        options_layout.addRow("Target PyQt version", self._pyqt_version_edit)
-
         self._console_edit = QCheckBox("Use console (Windows)",
                 whatsThis="Enable console output for Windows applications. "
                         "Console output will be enabled automatically if no "
@@ -141,12 +120,25 @@ class ApplicationPage(QWidget):
 
         layout.addLayout(options_layout, 0, 1)
 
+        # Extra space is needed before the application package editor.
+        layout.setRowMinimumHeight(1,
+                1.4 * QFontInfo(QGuiApplication.font()).pixelSize())
+
         self._package_edit = _ApplicationPackageEditor()
         self._package_edit.package_changed.connect(self._package_changed)
         package_edit_gb = QGroupBox(self._package_edit.title)
+        package_edit_gb.setFlat(True)
         package_edit_gb.setLayout(self._package_edit)
-        layout.addWidget(package_edit_gb, 1, 0, 1, 2)
-        layout.setRowStretch(1, 1)
+        layout.addWidget(package_edit_gb, 2, 0, 1, 2)
+
+        qmake = CollapsibleWidget("Additional qmake Configuration")
+        self._qmake_edit = QPlainTextEdit(
+                whatsThis="Any text entered here will be appended to the "
+                        "generated <tt>.pro</tt> that will be processed by "
+                        "<tt>qmake</tt>.",
+                textChanged=self._qmake_changed)
+        qmake.setWidget(self._qmake_edit)
+        layout.addWidget(qmake, 3, 0, 1, 2)
 
         self.setLayout(layout)
 
@@ -161,16 +153,6 @@ class ApplicationPage(QWidget):
         self._sys_path_edit.setText(project.sys_path)
         self._package_edit.configure(project.application_package, project)
 
-        blocked = self._py_version_edit.blockSignals(True)
-        self._py_version_edit.setCurrentIndex(
-                supported_python_versions.index(project.python_target_version))
-        self._py_version_edit.blockSignals(blocked)
-
-        blocked = self._pyqt_version_edit.blockSignals(True)
-        self._pyqt_version_edit.setCurrentIndex(
-                1 if project.application_is_pyqt5 else 0)
-        self._pyqt_version_edit.blockSignals(blocked)
-
         blocked = self._console_edit.blockSignals(True)
         self._console_edit.setCheckState(
                 Qt.Checked if project.application_is_console else Qt.Unchecked)
@@ -181,22 +163,9 @@ class ApplicationPage(QWidget):
                 Qt.Checked if project.application_is_bundle else Qt.Unchecked)
         self._bundle_edit.blockSignals(blocked)
 
-    def _py_version_changed(self, idx):
-        """ Invoked when the user changes the Python version number. """
-
-        self.project.python_target_version = supported_python_versions[idx]
-        self.project.modified = True
-
-        self.python_target_version_changed.emit()
-
-    def _pyqt_version_changed(self, idx):
-        """ Invoked when the user changes the PyQt version number. """
-
-        pyqt5 = (idx == 1)
-        self.project.application_is_pyqt5 = pyqt5
-        self.project.modified = True
-
-        self.pyqt_version_changed.emit(pyqt5)
+        blocked = self._qmake_edit.blockSignals(True)
+        self._qmake_edit.setPlainText(self._project.qmake_configuration)
+        self._qmake_edit.blockSignals(blocked)
 
     def _console_changed(self, state):
         """ Invoked when the user changes the console state. """
@@ -239,6 +208,12 @@ class ApplicationPage(QWidget):
 
         self.project.modified = True
 
+    def _qmake_changed(self):
+        """ Invoked when the user edits the qmake configuration. """
+
+        self.project.qmake_configuration = self._qmake_edit.toPlainText()
+        self.project.modified = True
+
 
 class _ApplicationPackageEditor(PackageEditor):
     """ A memory filesystem package editor for the application package. """
@@ -257,6 +232,8 @@ class _ApplicationPackageEditor(PackageEditor):
                         "and files that should be included in the "
                         "application.")
 
+        self.setContentsMargins(0, 0, 0, 0)
+
         self._project = None
 
     def get_root_dir(self):
@@ -267,13 +244,13 @@ class _ApplicationPackageEditor(PackageEditor):
 
         default = application_package.name
         if default is not None:
-            default = project.path_from_user(default)
+            default = project.project_path(default)
 
         root = QFileDialog.getExistingDirectory(self.parentWidget(),
                 self.title, default)
 
         if root != '':
-            application_package.name = project.path_to_user(root)
+            application_package.name = project.project_path(root)
 
         return root
 
