@@ -1,4 +1,4 @@
-# Copyright (c) 2019, Riverbank Computing Limited
+# Copyright (c) 2020, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,11 +26,11 @@
 
 import glob
 import os
-import struct
 import subprocess
 import sys
 
 from .user_exception import UserException
+from .version_number import VersionNumber
 
 
 class Platform:
@@ -39,6 +39,9 @@ class Platform:
     # The list of all platforms.
     all_platforms = []
 
+    # The name of the make executable.
+    make = 'make'
+    
     def __init__(self, full_name, name, archs):
         """ Initialise the object. """
 
@@ -66,18 +69,6 @@ class Platform:
 
         return name
 
-    def get_apple_sdk(self, message_handler):
-        """ Return the name of the Apple SDK. """
-
-        raise NotImplementedError
-
-    @property
-    def make(self):
-        """ The name of the make executable including any required path. """
-
-        # Platforms that can be hosts must reimplement this.
-        raise NotImplementedError
-
     @classmethod
     def platform(cls, name):
         """ Return the singleton Platform instance for a platform.  A
@@ -88,13 +79,14 @@ class Platform:
             if platform.name == name:
                 return platform
 
-        raise UserException("'{0}' is not a supported platform.".format(name))
+        raise UserException("'{0}' is not a supported platform".format(name))
 
     @staticmethod
     def run(*args, message_handler, capture=False):
         """ Run a command, optionally capturing stdout. """
 
-        message_handler.verbose_message("Running '{0}'".format(' '.join(args)))
+        message_handler.verbose_message(
+                "Running '{0}'.".format(' '.join(args)))
 
         detail = None
         stdout = []
@@ -122,10 +114,14 @@ class Platform:
             detail = str(e)
 
         if detail:
-            raise UserException("Execution failed: {}".format(detail))
+            raise UserException(
+                    "execution of '{0}' failed: {1}".format(args[0], detail))
 
         return ''.join(stdout).strip() if capture else None
 
+    def verify_as_target(self, message_handler):
+        """ Verify the platform as a target. """
+
 
 class Architecture:
     """ Encapsulate an architecture. """
@@ -149,26 +145,32 @@ class Architecture:
         """
 
         if name is None:
-            # Note that for Windows the default architecture is determined by
-            # the MSVC target.  On other platforms it is determined by the
-            # Python installation running this script.
-            if sys.platform == 'win32':
-                name = 'win-' + WindowsArchitecture.msvc_target()
-            else:
-                if sys.platform == 'darwin':
-                    name = 'macos'
-                elif sys.platform.startswith('linux'):
-                    name = 'linux'
-                else:
-                    raise UserException(
-                            "'{0}' is not a supported host platform.".format(
-                                    sys.platform))
+            from distutils.util import get_platform
 
-                name = '{0}-{1}'.format(name, 8 * struct.calcsize('P'))
-        elif name.startswith('osx-'):
-            # Map the deprecated values.  Such values can only come from the
-            # command line.
-            name = 'macos-' + name.split('-')[1]
+            parts = get_platform().split('-')
+            base_platform = parts[0]
+
+            if base_platform == 'linux':
+                name = 'linux'
+                size = '64' if parts[1] == 'x86_64' else '32'
+            elif base_platform == 'macosx':
+                name = 'macos'
+                size = '64' if parts[2] == 'x86_64' else '32'
+            elif base_platform in ('win32', 'win'):
+                name = 'win'
+
+                # The default architecture is determined by any MSVC target.
+                # If it is missing then this will be picked up when the target
+                # is verified.
+                size = WindowsArchitecture.msvc_target(optional=True)
+                if size is None:
+                    size = 64 if len(parts) == 2 and parts[1] == 'amd64' else 32
+            else:
+                raise UserException(
+                        "'{0}' is not a supported host platform".format(
+                                base_platform))
+
+            name = '{}-{}'.format(name, size)
 
         # Find the architecture instance.
         for arch in cls.all_architectures:
@@ -180,7 +182,7 @@ class Architecture:
                 return arch
 
         raise UserException(
-                "'{0}' is not a supported architecture.".format(name))
+                "'{0}' is not a supported architecture".format(name))
 
     def configure(self):
         """ Configure the architecture for building. """
@@ -212,7 +214,7 @@ class Architecture:
             if isinstance(targets, str):
                 # String targets can come from the project file (ie. the user)
                 # and so need to be validated.
-                if targets[0] == '!':
+                if targets.startswith('!'):
                     # Note that this assumes that the target is a platform
                     # rather than an architecture.  If this is incorrect then
                     # it is a bug in the meta-data somewhere.
@@ -231,28 +233,62 @@ class Architecture:
 
         return covered
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
         # This default implementation checks that the architectures are the
         # same.
         return target is self
 
+    def verify_as_host(self, target, message_handler):
+        """ Verify the architecture as a host for a given target. """
+
+        # Check we can host the target.
+        if not self.supported_target(target, message_handler):
+            raise UserException(
+                    "{0} is not a supported {1} development host".format(
+                            self.name, target.name))
+
+    def verify_as_target(self, message_handler):
+        """ Verify the architecture as a target. """
+
+        self.platform.verify_as_target(message_handler)
+
 
 class ApplePlatform(Platform):
     """ Encapsulate an Apple platform. """
 
-    @staticmethod
-    def find_sdk(sdk_name, message_handler):
-        """ Find an SDK to use. """
+    # The name of the SDK.
+    sdk_name = ''
 
-        sdk = Platform.run('xcrun', '--sdk', sdk_name, '--show-sdk-path',
-                message_handler=message_handler, capture=True)
+    # The prefix of the directory name of the SDK.
+    sdk_prefix = ''
 
-        if not sdk:
-            raise UserException("A valid SDK could not be found")
+    def verify_as_target(self, message_handler):
+        """ Verify the platform as a target. """
 
-        return sdk
+        super().verify_as_target(message_handler)
+
+        self.apple_sdk = self.run('xcrun', '--sdk', self.sdk_name,
+                '--show-sdk-path', message_handler=message_handler,
+                capture=True)
+
+        if not self.apple_sdk:
+            raise UserException(
+                    "a valid '{0}' SDK could not be found".format(
+                            self.sdk_name))
+
+        # Parse the version number.
+        version_str = os.path.basename(self.apple_sdk)
+
+        if version_str.startswith(self.sdk_prefix):
+            version_str = version_str[len(self.sdk_prefix):]
+
+        if version_str.endswith('.sdk'):
+            version_str = version_str[:-len('.sdk')]
+
+        self.apple_sdk_version = VersionNumber.parse_version_number(
+                version_str)
 
 
 # Define and implement the different platforms and architectures.  These should
@@ -261,133 +297,75 @@ class ApplePlatform(Platform):
 class AndroidArchitecture(Architecture):
     """ A base class for any Android architecture. """
 
-    def configure(self):
-        """ Configure the architecture for building. """
+    # The name of the Android platform's architecture ABI (as recognised by
+    # qmake).
+    android_abi = ''
 
-        # Configure the platform first.
-        self.platform.configure()
+    # The name of the Android platform's architecture.
+    android_platform_arch = ''
+
+    # The name of the Android toolchain's prefix.
+    android_toolchain_prefix = ''
+
+    # The architecture-specific clang prefix.
+    clang_prefix = ''
+
+    def verify_as_target(self, message_handler):
+        """ Verify the architecture as a target. """
+
+        super().verify_as_target(message_handler)
 
         # Set the various property values.
-        ndk_root = self.platform.ndk_root
+        ndk_root = self.platform.android_ndk_root
         android_api = self.platform.android_api
         toolchain_prefix = self.android_toolchain_prefix
         android_host = '{}-x86_64'.format(
                 'darwin' if sys.platform == 'darwin' else 'linux')
 
-        self.android_ndk_sysroot = os.path.join(ndk_root, 'platforms',
-                'android-{}'.format(android_api), self.android_platform_arch)
-
-        self._check_exists(self.android_ndk_sysroot)
-
-        # We use clang for r16 and later.
-        cflags = []
-
-        if self.platform.android_ndk_version >= (16, 0, 0):
-            self.android_toolchain_cc = '{}{}-clang'.format(self.clang_prefix,
-                    android_api)
-            toolchain_dir = 'llvm'
-        else:
-            # The architecture-neutral gcc flags.
-            cflags.append('--sysroot=' + self.android_ndk_sysroot)
-
-            cflags.extend(self.gcc_toolchain_cflags)
-
-            toolchain_version = self.platform.ndk_toolchain_version
-            self.android_toolchain_cc = toolchain_prefix + 'gcc'
-            toolchain_dir = toolchain_prefix + toolchain_version
-
-        self.android_toolchain_cflags = cflags
-
         # Check the toolchain bin directory.
         self.android_toolchain_bin = os.path.join(ndk_root, 'toolchains',
-                toolchain_dir, 'prebuilt', android_host, 'bin')
+                'llvm', 'prebuilt', android_host, 'bin')
 
-        self._check_exists(self.android_toolchain_bin)
+        self.platform.android_check_exists(self.android_toolchain_bin)
 
         # Check the compiler.
-        self._check_exists(
+        self.android_toolchain_cc = '{}{}-clang'.format(self.clang_prefix,
+                android_api)
+
+        self.platform.android_check_exists(
                 os.path.join(self.android_toolchain_bin,
                         self.android_toolchain_cc))
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
         # Android can never be a host.
         return False
 
-    @staticmethod
-    def _check_exists(name):
-        """ Raise an exception if something is missing from the NDK. """
-
-        if not os.path.exists(name):
-            raise UserException("'{0}' does not exist, make sure ANDROID_NDK_ROOT and ANDROID_NDK_PLATFORM are set correctly".format(name))
-
 
 class Android_arm_32(AndroidArchitecture):
     """ Encapsulate the Android 32-bit Arm architecture. """
 
-    @property
-    def android_platform_arch(self):
-        """ The name of the Android platform's architecture. """
-
-        return 'arch-arm'
-
-    @property
-    def android_toolchain_prefix(self):
-        """ The name of the Android toolchain's prefix. """
-
-        return 'arm-linux-androideabi-'
-
-    @property
-    def clang_prefix(self):
-        """ The architecture-specific clang prefix. """
-
-        return 'armv7a-linux-androideabi'
-
-    @property
-    def gcc_toolchain_cflags(self):
-        """ The architecture-specific gcc compiler flags. """
-
-        return ['-march=armv7-a', '-mfloat-abi=softfp', '-mfpu=vfp',
-                '-fno-builtin-memmove', '-mthumb']
+    # Archtecture-specific values.
+    android_abi = 'armeabi-v7a'
+    android_platform_arch = 'arch-arm'
+    android_toolchain_prefix = 'arm-linux-androideabi-'
+    clang_prefix = 'armv7a-linux-androideabi'
 
 
 class Android_arm_64(AndroidArchitecture):
     """ Encapsulate the Android 64-bit Arm architecture. """
 
-    @property
-    def android_platform_arch(self):
-        """ The name of the Android platform's architecture. """
-
-        return 'arch-arm64'
-
-    @property
-    def android_toolchain_prefix(self):
-        """ The name of the Android toolchain's prefix. """
-
-        return 'aarch64-linux-android-'
-
-    @property
-    def clang_prefix(self):
-        """ The architecture-specific clang prefix. """
-
-        return 'aarch64-linux-android'
-
-    @property
-    def gcc_toolchain_cflags(self):
-        """ The architecture-specific gcc compiler flags. """
-
-        # gcc is never used to for android-64.
-        return []
+    # Archtecture-specific values.
+    android_abi = 'arm64-v8a'
+    android_platform_arch = 'arch-arm64'
+    android_toolchain_prefix = 'aarch64-linux-android-'
+    clang_prefix = 'aarch64-linux-android'
 
 
 class Android(Platform):
     """ Encapsulate the Android platform. """
 
-    # The environment variables that should be set.
-    REQUIRED_ENV_VARS = ('ANDROID_NDK_ROOT', 'ANDROID_NDK_PLATFORM',
-            'ANDROID_SDK_ROOT')
-
     def __init__(self):
         """ Initialise the object. """
 
@@ -395,68 +373,82 @@ class Android(Platform):
                 [('android-32', Android_arm_32),
                         ('android-64', Android_arm_64)])
 
-    def configure(self):
-        """ Configure the platform for building. """
+    @staticmethod
+    def android_check_exists(name):
+        """ Raise an exception if something is missing from the NDK. """
 
-        for name in self.REQUIRED_ENV_VARS:
+        if not os.path.exists(name):
+            raise UserException(
+                    "'{0}' does not exist, make sure ANDROID_NDK_ROOT and "
+                    "ANDROID_NDK_PLATFORM are set correctly".format(name))
+
+    # The environment variables that should be set.
+    _REQUIRED_ENV_VARS = ('ANDROID_NDK_ROOT', 'ANDROID_NDK_PLATFORM',
+            'ANDROID_SDK_ROOT')
+
+    def verify_as_target(self, message_handler):
+        """ Verify the platform as a target. """
+
+        super().verify_as_target(message_handler)
+
+        # Verify required environment variables.
+        for name in self._REQUIRED_ENV_VARS:
             if name not in os.environ:
                 raise UserException(
-                        "The {0} environment variable must be set.".format(
+                        "the {0} environment variable must be set".format(
                                 name))
 
-        self.ndk_root = os.environ['ANDROID_NDK_ROOT']
-        self.sdk_root = os.environ['ANDROID_SDK_ROOT']
+        self.android_ndk_root = os.environ['ANDROID_NDK_ROOT']
 
-        self._original_toolchain_version = os.environ.get(
-                'ANDROID_NDK_TOOLCHAIN_VERSION')
+        self.android_ndk_sysroot = os.path.join(self.android_ndk_root,
+                'sysroot')
+        self.android_check_exists(self.android_ndk_sysroot)
 
-        if self._original_toolchain_version is None:
-            # This is only used by a gcc toolchain.
-            self.ndk_toolchain_version = '4.9'
-            os.environ['ANDROID_NDK_TOOLCHAIN_VERSION'] = self.ndk_toolchain_version
-        else:
-            self.ndk_toolchain_version = self._original_toolchain_version
+        # Verify the NDK revision.
+        self.android_ndk_version = self._get_ndk_version()
+        if self.android_ndk_version is None:
+            raise UserException("unable to determine the NDK revision")
 
-        self._set_ndk_version()
-        self._set_sdk_version()
-        self._set_api()
+        # Require a minimum of r19 so that we can assume the compiler is clang
+        # and that it works properly.
+        revision = self.android_ndk_version.major
+        if revision < 19:
+            raise UserException("NDK r19 or later is required")
 
-        # Blacklist r11-13 as they have problems finding standard library .h
-        # files.  It is probably something simple, like a missing --sysroot
-        # flag.  Also blacklist r16-18 to avoid having to deal with
-        # make-standalone-toolchain.sh for clang.
-        revision = self.android_ndk_version[0]
-        if revision in (11, 12, 13, 16, 17, 18):
-            raise UserException("NDK r{0} is not supported.".format(revision))
+        # Issue a warning for untested NDK revision.
+        if revision > 21:
+            message_handler.warning(
+                    "versions of the NDK later than r21 are untested")
 
-        # Force the gcc toolchain for r15 and earlier.
-        self._original_qmakespec = os.environ.get('QMAKESPEC')
-        os.environ['QMAKESPEC'] = 'android-g++' if revision <= 15 else 'android-clang'
+        # Verify the SDK version.
+        self.android_sdk_version = self._get_sdk_version()
+        if self.android_sdk_version is None:
+            raise UserException("unable to determine the SDK version number")
 
-    def deconfigure(self):
-        """ Deconfigure the platform for building. """
+        if self.android_sdk_version < (26, 1, 1):
+            message_handler.warning(
+                    "versions of the SDK earlier than v26.1.1 are untested")
 
-        if self._original_qmakespec is None:
-            del os.environ['QMAKESPEC']
-        else:
-            os.environ['QMAKESPEC'] = self._original_qmakespec
+        if self.android_sdk_version > (26, 1, 1):
+            message_handler.warning(
+                    "versions of the SDK later than v26.1.1 are untested")
 
-        if self._original_toolchain_version is None:
-            del os.environ['ANDROID_NDK_TOOLCHAIN_VERSION']
-        else:
-            os.environ['ANDROID_NDK_TOOLCHAIN_VERSION'] = self._original_toolchain_version
+        # Verify the API.
+        self.android_api = self._get_api()
+        if self.android_api is None:
+            raise UserException(
+                    "unable to determine the API level from the "
+                    "ANDROID_NDK_PLATFORM environment variable")
 
-    def _set_api(self):
-        """ Set the number of the Android API. """
-
-        api = None
+    def _get_api(self):
+        """ Return the number of the Android API. """
 
         ndk_platform = os.environ['ANDROID_NDK_PLATFORM']
 
-        if not os.path.isdir(os.path.join(self.ndk_root, 'platforms', ndk_platform)):
+        if not os.path.isdir(os.path.join(self.android_ndk_root, 'platforms', ndk_platform)):
             raise UserException(
-                    "NDK r{0} does not support {1}.".format(
-                            self.android_ndk_version[0], ndk_platform))
+                    "NDK r{0} does not support {1}".format(
+                            self.android_ndk_version.major, ndk_platform))
 
         parts = ndk_platform.split('-')
 
@@ -466,90 +458,74 @@ class Android(Platform):
             except ValueError:
                 api = None
 
-        if api is None:
-            raise UserException(
-                    "Unable to determine the API level from the ANDROID_NDK_PLATFORM environment variable.")
-
-        self.android_api = api
-
-    def _set_ndk_version(self):
-        """ Set the version number of the NDK. """
-
-        # source.properties is available from r11.
-        source_properties = os.path.join(self.ndk_root, 'source.properties')
-        if os.path.isfile(source_properties):
-            self.android_ndk_version = self._get_version(source_properties)
-        else:
-            # RELEASE.TXT is available in r10 and earlier.
-            self.android_ndk_version = None
-
-            release_txt = os.path.join(self.ndk_root, 'RELEASE.TXT')
-            if os.path.isfile(release_txt):
-                with open(release_txt) as f:
-                    for line in f:
-                        if line.startswith('r'):
-                            line = line[1:]
-                            for i, ch in enumerate(line):
-                                if not ch.isdigit():
-                                    line = line[:i]
-                                    break
-
-                            try:
-                                # Note that we ignore the minor letter.
-                                self.android_ndk_version = (int(line), 0, 0)
-                                break
-                            except ValueError:
-                                pass
-
-    def _set_sdk_version(self):
-        """ Set the version number of the SDK. """
-
-        # Assume that source.properties should be available.
-        source_properties = os.path.join(self.sdk_root, 'tools',
-                'source.properties')
-
-        if not os.path.exists(source_properties):
-            raise UserException("'{0}' does not exist, make sure ANDROID_SDK_ROOT is set correctly".format(source_properties))
-
-        self.android_sdk_version = self._get_version(source_properties)
+        return api
 
     @staticmethod
     def _get_version(source_properties):
-        """ Get the 3-tuple version number of a source.properties file. """
-
-        version = None
+        """ Get the version number of a source.properties file. """
 
         with open(source_properties) as f:
             for line in f:
                 line = line.replace(' ', '')
                 parts = line.split('=')
                 if parts[0] == 'Pkg.Revision' and len(parts) == 2:
-                    version_parts = parts[1].split('.')
-
-                    if len(version_parts) <= 3:
-                        while len(version_parts) < 3:
-                            version_parts.append('0')
-
-                        try:
-                            major = int(version_parts[0])
-                            minor = int(version_parts[1])
-                            maint = int(version_parts[2])
-
-                            version = (major, minor, maint)
-                        except ValueError:
-                            pass
-
+                    version = VersionNumber.parse_version_number(parts[1])
                     break
+            else:
+                version = None
 
         return version
 
+    def _get_ndk_version(self):
+        """ Return the version number of the NDK. """
+
+        # source.properties is available from r11.
+        source_properties = os.path.join(self.android_ndk_root,
+                'source.properties')
+        if os.path.isfile(source_properties):
+            return self._get_version(source_properties)
+
+        # RELEASE.TXT is available in r10 and earlier.
+        release_txt = os.path.join(self.android_ndk_root, 'RELEASE.TXT')
+        if os.path.isfile(release_txt):
+            with open(release_txt) as f:
+                for line in f:
+                    if line.startswith('r'):
+                        line = line[1:]
+                        for i, ch in enumerate(line):
+                            if not ch.isdigit():
+                                line = line[:i]
+                                break
+
+                        try:
+                            # Note that we ignore the minor letter.
+                            return VersionNumber(int(line))
+                        except UserException:
+                            pass
+
+        return None
+
+    def _get_sdk_version(self):
+        """ Return the version number of the SDK. """
+
+        # Assume that source.properties should be available.
+        source_properties = os.path.join(os.environ['ANDROID_SDK_ROOT'],
+                'tools', 'source.properties')
+
+        if not os.path.exists(source_properties):
+            raise UserException(
+                    "'{0}' does not exist, make sure ANDROID_SDK_ROOT is set "
+                    "correctly".format(source_properties))
+
+        return self._get_version(source_properties)
+
 Android()
 
 
 class iOS_arm_64(Architecture):
     """ Encapsulate the ios 64-bit Arm architecture. """
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
         # iOS can never be a host.
@@ -558,35 +534,16 @@ class iOS_arm_64(Architecture):
 
 class iOS(ApplePlatform):
     """ Encapsulate the iOS platform. """
-    
+
+    # Platform-specific values.
+    sdk_name = 'iphoneos'
+    sdk_prefix = 'iPhoneOS'
+
     def __init__(self):
         """ Initialise the object. """
         
         super().__init__("iOS", 'ios', [('ios-64', iOS_arm_64)])
 
-        self._original_deployment_target = os.environ.get(
-                'IPHONEOS_DEPLOYMENT_TARGET')
-
-    def configure(self):
-        """ Configure the platform for building. """
-
-        if self._original_deployment_target is None:
-            # If not set then use the value that Qt uses.
-            os.environ['IPHONEOS_DEPLOYMENT_TARGET'] = '8.0'
-
-    def deconfigure(self):
-        """ Deconfigure the platform for building. """
-
-        if self._original_deployment_target is None:
-            del os.environ['IPHONEOS_DEPLOYMENT_TARGET']
-        else:
-            os.environ['IPHONEOS_DEPLOYMENT_TARGET'] = self._original_deployment_target
-
-    def get_apple_sdk(self, message_handler):
-        """ The name of the iOS SDK. """
-
-        return self.find_sdk('iphoneos', message_handler)
-
 iOS()
 
 
@@ -599,13 +556,13 @@ class Linux_x86_32(Architecture):
 class Linux_x86_64(Architecture):
     """ Encapsulate the Linux 64-bit x86 architecture. """
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
         if target.platform.name == 'android':
             return True
 
-        return super().supported_target(target)
+        return super().supported_target(target, message_handler)
 
 
 class Linux(Platform):
@@ -617,91 +574,76 @@ class Linux(Platform):
         super().__init__("Linux", 'linux',
                 [('linux-32', Linux_x86_32), ('linux-64', Linux_x86_64)])
 
-    @property
-    def make(self):
-        """ The name of the make executable including any required path. """
-
-        return 'make'
-
 Linux()
 
 
 class macOS_x86_64(Architecture):
     """ Encapsulate the macOS 64-bit x86 architecture. """
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
         if target.platform.name in ('android', 'ios'):
             return True
 
-        return super().supported_target(target)
+        return super().supported_target(target, message_handler)
 
 
 class macOS(ApplePlatform):
     """ Encapsulate the macOS platform. """
-    
+
+    # Platform-specific values.
+    sdk_name = 'macosx'
+    sdk_prefix = 'MacOSX'
+
     def __init__(self):
         """ Initialise the object. """
         
         super().__init__("macOS", 'macos', [('macos-64', macOS_x86_64)])
 
-        self._original_deployment_target = os.environ.get(
-                'MACOSX_DEPLOYMENT_TARGET')
-
-    def configure(self):
-        """ Configure the platform for building. """
-
-        if self._original_deployment_target is None:
-            # If not set then use the value that Qt uses.
-            os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.10'
-
-    def deconfigure(self):
-        """ Deconfigure the platform for building. """
-
-        if self._original_deployment_target is None:
-            del os.environ['MACOSX_DEPLOYMENT_TARGET']
-        else:
-            os.environ['MACOSX_DEPLOYMENT_TARGET'] = self._original_deployment_target
-
-    def get_apple_sdk(self, message_handler):
-        """ The name of the macOS SDK. """
-
-        return self.find_sdk('macosx', message_handler)
-
-    @property
-    def make(self):
-        """ The name of the make executable including any required path. """
-
-        return 'make'
-
 macOS()
 
 
 class WindowsArchitecture(Architecture):
     """ Encapsulate any Windows x86 architecture. """
 
-    def supported_target(self, target):
+    def supported_target(self, target, message_handler):
         """ Check that this architecture can host a target architecture. """
 
-        return target.platform is self.platform
+        if target.platform.name == 'android':
+            message_handler.warning(
+                    "using Windows to host Android deployment is untested")
+
+            return True
+
+        return super().supported_target(target, message_handler)
 
     @staticmethod
-    def msvc_target():
+    def msvc_target(optional=False):
         """ Return '32' or 64' depending the architecture being targeted by
         MSVC and raise an exception if a supported version of MSVC could not be
         found.
         """
 
-        # MSVC2015 is v14 and MSVC2017 is v15.
-        vs_major = os.environ.get('VisualStudioVersion', '0.0').split('.')[0]
+        # MSVC2015 is v14, MSVC2017 is v15 and MSVC2019 is v16.
+        vs_version = os.environ.get('VisualStudioVersion', '0.0')
+        vs_major = vs_version.split('.')[0]
 
-        if vs_major == '15':
-            is_32 = (os.environ.get('VSCMD_ARG_TGT_ARCH') != 'x64')
-        elif vs_major == '14':
+        if vs_major == '0':
+            if optional:
+                return None
+
+            raise UserException("unable to detect any MSVC compiler")
+
+        if vs_major == '14':
             is_32 = (os.environ.get('Platform') != 'X64')
+        elif vs_major in ('15', '16'):
+            is_32 = (os.environ.get('VSCMD_ARG_TGT_ARCH') != 'x64')
         else:
-            raise UserException("Unable to detect MSVC2015 or MSVC2017.")
+            if optional:
+                return None
+
+            raise UserException("MSVC v{0} is unsupported".format(vs_version))
 
         return '32' if is_32 else '64'
 
@@ -709,26 +651,33 @@ class WindowsArchitecture(Architecture):
 class Windows_x86_32(WindowsArchitecture):
     """ Encapsulate the Windows 32-bit x86 architecture. """
 
-    def configure(self):
-        """ Configure the platform for building. """
+    def verify_as_target(self, message_handler):
+        """ Verify the architecture as a target. """
+
+        super().verify_as_target(message_handler)
 
         if self.msvc_target() != '32':
-            raise UserException("MSVC is not configured for a 32-bit target.")
+            raise UserException("MSVC is not configured for a 32-bit target")
 
 
 class Windows_x86_64(WindowsArchitecture):
     """ Encapsulate the Windows 64-bit x86 architecture. """
 
-    def configure(self):
-        """ Configure the platform for building. """
+    def verify_as_target(self, message_handler):
+        """ Verify the architecture as a target. """
+
+        super().verify_as_target(message_handler)
 
         if self.msvc_target() != '64':
-            raise UserException("MSVC is not configured for a 64-bit target.")
+            raise UserException("MSVC is not configured for a 64-bit target")
 
 
 
 class Windows(Platform):
     """ Encapsulate the Windows platform. """
+
+    # The name of the make executable.
+    make = 'nmake'
     
     def __init__(self):
         """ Initialise the object. """
@@ -739,12 +688,9 @@ class Windows(Platform):
     def exe(self, name):
         """ Convert a generic executable name to a host-specific version. """
 
-        return name + '.exe'
+        if not name.endswith('.exe'):
+            name += '.exe'
 
-    @property
-    def make(self):
-        """ The name of the make executable including any required path. """
-
-        return 'nmake'
+        return name
 
 Windows()
